"""
@author: Andrius Bernatavicius, 2019
"""

from __future__ import print_function
from src.env    import VrepEnvironment
from src.agents import Pioneer
from src.disp   import Display
import settings
import time, argparse
import matplotlib.pyplot as plt

###########
#
###########

def loop(agent):
    
    """
    Agent control loop
    
    Your code goes here
    """
    agent.change_velocity([5,5])


##########
#
##########

if __name__ == "__main__":

    plt.ion()

    # Initialize and start environment
    environment = VrepEnvironment(settings.SCENES + '/room_static.ttt')  # Open the file containing our scene (robot and its environment)
    environment.connect()        # Connect python to the simulator's remote API

    # Create the robot object and SLAM display
    agent   = Pioneer(environment)
    display = Display(agent, False) 

    print('\nDemonstration of Simultaneous Localization and Mapping using V-REP robot simulation software. \nPress "CTRL+C" to exit.\n')
    start = time.time()
    step  = 0
    done  = False
    environment.start_simulation()
    time.sleep(1)

    try:    
        while step < settings.simulation_steps and not done:
            display.update()                      # Update the SLAM display
            loop(agent)                           # Control loop
            step += 1
    except KeyboardInterrupt:
        print('\n\nInterrupted! Time: {}s'.format(time.time()-start))
        
        
    display.close()
    environment.stop_simulation()
    environment.disconnect()
    
    
    
