local bwUtils=require('bwUtils')

function removeFromPluginRepresentation()

end

function updatePluginRepresentation()

end

function setShapeSize(h,x,y,z)
    local r,mmin=simGetObjectFloatParameter(h,sim_objfloatparam_objbbox_min_x)
    local r,mmax=simGetObjectFloatParameter(h,sim_objfloatparam_objbbox_max_x)
    local sx=mmax-mmin
    local r,mmin=simGetObjectFloatParameter(h,sim_objfloatparam_objbbox_min_y)
    local r,mmax=simGetObjectFloatParameter(h,sim_objfloatparam_objbbox_max_y)
    local sy=mmax-mmin
    local r,mmin=simGetObjectFloatParameter(h,sim_objfloatparam_objbbox_min_z)
    local r,mmax=simGetObjectFloatParameter(h,sim_objfloatparam_objbbox_max_z)
    local sz=mmax-mmin
    simScaleObject(h,x/sx,y/sy,z/sz)
end

function getDefaultInfoForNonExistingFields(info)
    if not info['version'] then
        info['version']=_MODELVERSION_
    end
    if not info['subtype'] then
        info['subtype']='A'
    end
    if not info['velocity'] then
        info['velocity']=0.1
    end
    if not info['acceleration'] then
        info['acceleration']=0.01
    end
    if not info['length'] then
        info['length']=1
    end
    if not info['width'] then
        info['width']=0.3
    end
    if not info['height'] then
        info['height']=0.1
    end
    if not info['borderHeight'] then
        info['borderHeight']=0.2
    end
    if not info['bitCoded'] then
        info['bitCoded']=1+2+4+8
    end
    if not info['wallThickness'] then
        info['wallThickness']=0.005
    end
    if not info['stopRequests'] then
        info['stopRequests']={}
    end
end

function readInfo()
    local data=simReadCustomDataBlock(model,'CONVEYOR_CONF')
    if data then
        data=simUnpackTable(data)
    else
        data={}
    end
    getDefaultInfoForNonExistingFields(data)
    return data
end

function writeInfo(data)
    if data then
        simWriteCustomDataBlock(model,'CONVEYOR_CONF',simPackTable(data))
    else
        simWriteCustomDataBlock(model,'CONVEYOR_CONF','')
    end
end
function setColor(red,green,blue,spec)
    simSetShapeColor(middleParts[2],nil,sim_colorcomponent_ambient_diffuse,{red,green,blue})
    simSetShapeColor(middleParts[2],nil,sim_colorcomponent_specular,{spec,spec,spec})
    simSetShapeColor(endParts[1],nil,sim_colorcomponent_ambient_diffuse,{red,green,blue})
    simSetShapeColor(endParts[1],nil,sim_colorcomponent_specular,{spec,spec,spec})
    simSetShapeColor(endParts[2],nil,sim_colorcomponent_ambient_diffuse,{red,green,blue})
    simSetShapeColor(endParts[2],nil,sim_colorcomponent_specular,{spec,spec,spec})
end

function getColor()
    local r,rgb=simGetShapeColor(middleParts[2],nil,sim_colorcomponent_ambient_diffuse)
    local r,spec=simGetShapeColor(middleParts[2],nil,sim_colorcomponent_specular)
    return rgb[1],rgb[2],rgb[3],(spec[1]+spec[2]+spec[3])/3
end

function updateConveyor()
    local conf=readInfo()
    local length=conf['length']
    local width=conf['width']
    local height=conf['height']
    local borderHeight=conf['borderHeight']
    local bitCoded=conf['bitCoded']
    local wt=conf['wallThickness']
    local re=simBoolAnd32(bitCoded,16)==0
---[[
    simSetObjectPosition(rotJoints[1],model,{0,-length*0.5,-height*0.5})
    simSetObjectPosition(rotJoints[2],model,{0,length*0.5,-height*0.5})

    setShapeSize(middleParts[1],width,length,height)
    setShapeSize(middleParts[2],width,length,0.001)
    setShapeSize(middleParts[3],width,length,height)
    simSetObjectPosition(middleParts[1],model,{0,0,-height*0.5})
    simSetObjectPosition(middleParts[2],model,{0,0,-0.0005})
    simSetObjectPosition(middleParts[3],model,{0,0,-height*0.5})

    setShapeSize(endParts[1],width,0.083148*height/0.2,0.044443*height/0.2)
    simSetObjectPosition(endParts[1],model,{0,-length*0.5-0.5*0.083148*height/0.2,-0.044443*height*0.5/0.2})

    setShapeSize(endParts[2],width,0.083148*height/0.2,0.044443*height/0.2)
    simSetObjectPosition(endParts[2],model,{0,length*0.5+0.5*0.083148*height/0.2,-0.044443*height*0.5/0.2})

    setShapeSize(endParts[3],width,height*0.5,height)
    simSetObjectPosition(endParts[3],model,{0,-length*0.5-0.25*height,-height*0.5})

    setShapeSize(endParts[4],width,height*0.5,height)
    simSetObjectPosition(endParts[4],model,{0,length*0.5+0.25*height,-height*0.5})

    for i=5,6,1 do
        setShapeSize(endParts[i],height,height,width)
    end

    setShapeSize(sides[1],wt,length,height+2*borderHeight)
    setShapeSize(sides[2],wt,length,height+2*borderHeight)
    setShapeSize(sides[4],width+2*wt,height*0.5+1*borderHeight,height+2*borderHeight)
    setShapeSize(sides[3],width+2*wt,height*0.5+1*borderHeight,height+2*borderHeight)
    simSetObjectPosition(sides[4],model,{0,-(length+height*0.5+borderHeight)*0.5,-height*0.5})
    simSetObjectPosition(sides[3],model,{0,(length+height*0.5+borderHeight)*0.5,-height*0.5})
    simSetObjectPosition(sides[1],model,{-(width+wt)*0.5,0,-height*0.5})
    simSetObjectPosition(sides[2],model,{(width+wt)*0.5,0,-height*0.5})

    if re then
        simSetObjectInt32Parameter(endParts[1],sim_objintparam_visibility_layer,1)
        simSetObjectInt32Parameter(endParts[2],sim_objintparam_visibility_layer,1)
        simSetObjectInt32Parameter(endParts[3],sim_objintparam_visibility_layer,1)
        simSetObjectInt32Parameter(endParts[4],sim_objintparam_visibility_layer,1)
        simSetObjectInt32Parameter(endParts[5],sim_objintparam_visibility_layer,256)
        simSetObjectInt32Parameter(endParts[6],sim_objintparam_visibility_layer,256)
        simSetObjectInt32Parameter(endParts[5],sim_shapeintparam_respondable,1)
        simSetObjectInt32Parameter(endParts[6],sim_shapeintparam_respondable,1)
    else
        simSetObjectInt32Parameter(endParts[1],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(endParts[2],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(endParts[3],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(endParts[4],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(endParts[5],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(endParts[6],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(endParts[5],sim_shapeintparam_respondable,0)
        simSetObjectInt32Parameter(endParts[6],sim_shapeintparam_respondable,0)
    end

    if simBoolAnd32(bitCoded,1)~=0 then
        simSetObjectInt32Parameter(sides[1],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(sides[1],sim_shapeintparam_respondable,0)
        simSetObjectSpecialProperty(sides[1],0)
        simSetObjectProperty(sides[1],sim_objectproperty_dontshowasinsidemodel)
    else
        simSetObjectInt32Parameter(sides[1],sim_objintparam_visibility_layer,1+256)
        simSetObjectInt32Parameter(sides[1],sim_shapeintparam_respondable,1)
        simSetObjectSpecialProperty(sides[1],sim_objectspecialproperty_collidable+sim_objectspecialproperty_measurable+sim_objectspecialproperty_detectable_all+sim_objectspecialproperty_renderable)
        simSetObjectProperty(sides[1],sim_objectproperty_selectable+sim_objectproperty_selectmodelbaseinstead)
    end
    if simBoolAnd32(bitCoded,2)~=0 then
        simSetObjectInt32Parameter(sides[2],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(sides[2],sim_shapeintparam_respondable,0)
        simSetObjectSpecialProperty(sides[2],0)
        simSetObjectProperty(sides[2],sim_objectproperty_dontshowasinsidemodel)
    else
        simSetObjectInt32Parameter(sides[2],sim_objintparam_visibility_layer,1+256)
        simSetObjectInt32Parameter(sides[2],sim_shapeintparam_respondable,1)
        simSetObjectSpecialProperty(sides[2],sim_objectspecialproperty_collidable+sim_objectspecialproperty_measurable+sim_objectspecialproperty_detectable_all+sim_objectspecialproperty_renderable)
        simSetObjectProperty(sides[2],sim_objectproperty_selectable+sim_objectproperty_selectmodelbaseinstead)
    end
    if simBoolAnd32(bitCoded,4)~=0 or (not re) then
        simSetObjectInt32Parameter(sides[3],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(sides[3],sim_shapeintparam_respondable,0)
        simSetObjectSpecialProperty(sides[3],0)
        simSetObjectProperty(sides[3],sim_objectproperty_dontshowasinsidemodel)
    else
        simSetObjectInt32Parameter(sides[3],sim_objintparam_visibility_layer,1+256)
        simSetObjectInt32Parameter(sides[3],sim_shapeintparam_respondable,1)
        simSetObjectSpecialProperty(sides[3],sim_objectspecialproperty_collidable+sim_objectspecialproperty_measurable+sim_objectspecialproperty_detectable_all+sim_objectspecialproperty_renderable)
        simSetObjectProperty(sides[3],sim_objectproperty_selectable+sim_objectproperty_selectmodelbaseinstead)
    end
    if simBoolAnd32(bitCoded,8)~=0 or (not re) then
        simSetObjectInt32Parameter(sides[4],sim_objintparam_visibility_layer,0)
        simSetObjectInt32Parameter(sides[4],sim_shapeintparam_respondable,0)
        simSetObjectSpecialProperty(sides[4],0)
        simSetObjectProperty(sides[4],sim_objectproperty_dontshowasinsidemodel)
    else
        simSetObjectInt32Parameter(sides[4],sim_objintparam_visibility_layer,1+256)
        simSetObjectInt32Parameter(sides[4],sim_shapeintparam_respondable,1)
        simSetObjectSpecialProperty(sides[4],sim_objectspecialproperty_collidable+sim_objectspecialproperty_measurable+sim_objectspecialproperty_detectable_all+sim_objectspecialproperty_renderable)
        simSetObjectProperty(sides[4],sim_objectproperty_selectable+sim_objectproperty_selectmodelbaseinstead)
    end

    if simBoolAnd32(bitCoded,32)==0 then
        local textureID=simGetShapeTextureId(textureHolder)
        simSetShapeTexture(middleParts[2],textureID,sim_texturemap_plane,12,{0.04,0.04})
        simSetShapeTexture(endParts[1],textureID,sim_texturemap_plane,12,{0.04,0.04})
        simSetShapeTexture(endParts[2],textureID,sim_texturemap_plane,12,{0.04,0.04})
    else
        simSetShapeTexture(middleParts[2],-1,sim_texturemap_plane,12,{0.04,0.04})
        simSetShapeTexture(endParts[1],-1,sim_texturemap_plane,12,{0.04,0.04})
        simSetShapeTexture(endParts[2],-1,sim_texturemap_plane,12,{0.04,0.04})
    end
--]]    
end

function getAvailableSensors()
    local l=simGetObjectsInTree(sim_handle_scene,sim_handle_all,0)
    local retL={}
    for i=1,#l,1 do
        local data=simReadCustomDataBlock(l[i],'XYZ_BINARYSENSOR_INFO')
        if data then
            retL[#retL+1]={simGetObjectName(l[i]),l[i]}
        end
    end
    return retL
end

function getAvailableMasterConveyors()
    local l=simGetObjectsInTree(sim_handle_scene,sim_handle_all,0)
    local retL={}
    for i=1,#l,1 do
        if l[i]~=model then
            local data=simReadCustomDataBlock(l[i],'CONVEYOR_CONF')
            if data then
                retL[#retL+1]={simGetObjectName(l[i]),l[i]}
            end
        end
    end
    return retL
end

function lengthChange(ui,id,newVal)
    local conf=readInfo()
    local l=tonumber(newVal)
    if l then
        l=l*0.001
        if l<0.01 then l=0.01 end
        if l>5 then l=5 end
        if l~=conf['length'] then
            modified=true
            conf['length']=l
            writeInfo(conf)
            updateConveyor()
        end
    end
    simExtCustomUI_setEditValue(ui,2,string.format("%.0f",conf['length']/0.001),true)
end

function widthChange(ui,id,newVal)
    local conf=readInfo()
    local w=tonumber(newVal)
    if w then
        w=w*0.001
        if w<0.01 then w=0.01 end
        if w>2 then w=2 end
        if w~=conf['width'] then
            modified=true
            conf['width']=w
            writeInfo(conf)
            updateConveyor()
        end
    end
    simExtCustomUI_setEditValue(ui,4,string.format("%.0f",conf['width']/0.001),true)
end

function heightChange(ui,id,newVal)
    local conf=readInfo()
    local w=tonumber(newVal)
    if w then
        w=w*0.001
        if w<0.01 then w=0.01 end
        if w>1 then w=1 end
        if w~=conf['height'] then
            modified=true
            conf['height']=w
            writeInfo(conf)
            updateConveyor()
        end
    end
    simExtCustomUI_setEditValue(ui,20,string.format("%.0f",conf['height']/0.001),true)
end

function borderHeightChange(ui,id,newVal)
    local conf=readInfo()
    local w=tonumber(newVal)
    if w then
        w=w*0.001
        if w<0.005 then w=0.005 end
        if w>0.2 then w=0.2 end
        if w~=conf['borderHeight'] then
            modified=true
            conf['borderHeight']=w
            writeInfo(conf)
            updateConveyor()
        end
    end
    simExtCustomUI_setEditValue(ui,21,string.format("%.0f",conf['borderHeight']/0.001),true)
end

function wallThicknessChange(ui,id,newVal)
    local conf=readInfo()
    local w=tonumber(newVal)
    if w then
        w=w*0.001
        if w<0.001 then w=0.001 end
        if w>0.02 then w=0.02 end
        if w~=conf['wallThickness'] then
            modified=true
            conf['wallThickness']=w
            writeInfo(conf)
            updateConveyor()
        end
    end
    simExtCustomUI_setEditValue(ui,26,string.format("%.0f",conf['wallThickness']/0.001),true)
end

function leftSideOpenClicked(ui,id,newVal)
    local conf=readInfo()
    conf['bitCoded']=simBoolOr32(conf['bitCoded'],1)
    if newVal==0 then
        conf['bitCoded']=conf['bitCoded']-1
    end
    modified=true
    writeInfo(conf)
    updateConveyor()
end

function rightSideOpenClicked(ui,id,newVal)
    local conf=readInfo()
    conf['bitCoded']=simBoolOr32(conf['bitCoded'],2)
    if newVal==0 then
        conf['bitCoded']=conf['bitCoded']-2
    end
    modified=true
    writeInfo(conf)
    updateConveyor()
end

function frontSideOpenClicked(ui,id,newVal)
    local conf=readInfo()
    conf['bitCoded']=simBoolOr32(conf['bitCoded'],4)
    if newVal==0 then
        conf['bitCoded']=conf['bitCoded']-4
    end
    modified=true
    writeInfo(conf)
    updateConveyor()
end

function backSideOpenClicked(ui,id,newVal)
    local conf=readInfo()
    conf['bitCoded']=simBoolOr32(conf['bitCoded'],8)
    if newVal==0 then
        conf['bitCoded']=conf['bitCoded']-8
    end
    modified=true
    writeInfo(conf)
    updateConveyor()
end

function triggerStopChange_callback(ui,id,newIndex)
    local sens=comboStopTrigger[newIndex+1][2]
    bwUtils.setReferencedObjectHandle(model,REF_STOP_SIG,sens)
    if bwUtils.getReferencedObjectHandle(model,REF_START_SIG)==sens then
        bwUtils.setReferencedObjectHandle(model,REF_START_SIG,-1)
    end
    modified=true
    updateStartStopTriggerComboboxes()
end

function triggerStartChange_callback(ui,id,newIndex)
    local sens=comboStartTrigger[newIndex+1][2]
    bwUtils.setReferencedObjectHandle(model,REF_START_SIG,sens)
    if bwUtils.getReferencedObjectHandle(model,REF_STOP_SIG)==sens then
        bwUtils.setReferencedObjectHandle(model,REF_STOP_SIG,-1)
    end
    modified=true
    updateStartStopTriggerComboboxes()
end

function masterChange_callback(ui,id,newIndex)
    local sens=comboMaster[newIndex+1][2]
    bwUtils.setReferencedObjectHandle(model,REF_MASTER_CONV,sens) -- master
    updateMasterCombobox()
    updateEnabledDisabledItems()
    modified=true
end

function updateStartStopTriggerComboboxes()
    local c=readInfo()
    local loc=getAvailableSensors()
    comboStopTrigger=customUi_populateCombobox(ui,100,loc,{},bwUtils.getObjectNameOrNone(bwUtils.getReferencedObjectHandle(model,REF_STOP_SIG)),true,{{'<NONE>',-1}})
    comboStartTrigger=customUi_populateCombobox(ui,101,loc,{},bwUtils.getObjectNameOrNone(bwUtils.getReferencedObjectHandle(model,REF_START_SIG)),true,{{'<NONE>',-1}})
end

function updateMasterCombobox()
    local c=readInfo()
    local loc=getAvailableMasterConveyors()
    comboMaster=customUi_populateCombobox(ui,102,loc,{},bwUtils.getObjectNameOrNone(bwUtils.getReferencedObjectHandle(model,REF_MASTER_CONV)),true,{{'<NONE>',-1}})
end

function updateEnabledDisabledItems()
    if ui then
        local c=readInfo()
        local re=simBoolAnd32(c['bitCoded'],16)==0
        local simStopped=simGetSimulationState()==sim_simulation_stopped
        simExtCustomUI_setEnabled(ui,2,simStopped,true)
        simExtCustomUI_setEnabled(ui,4,simStopped,true)
        simExtCustomUI_setEnabled(ui,5,simStopped,true)
        simExtCustomUI_setEnabled(ui,6,simStopped,true)
        simExtCustomUI_setEnabled(ui,7,simStopped,true)
        simExtCustomUI_setEnabled(ui,8,simStopped,true)
        simExtCustomUI_setEnabled(ui,20,simStopped,true)
        simExtCustomUI_setEnabled(ui,21,simStopped,true)
        simExtCustomUI_setEnabled(ui,22,simStopped,true)
        simExtCustomUI_setEnabled(ui,23,simStopped,true)
        simExtCustomUI_setEnabled(ui,24,simStopped and re,true)
        simExtCustomUI_setEnabled(ui,25,simStopped and re,true)
        simExtCustomUI_setEnabled(ui,26,simStopped,true)
        simExtCustomUI_setEnabled(ui,27,simStopped,true)
        simExtCustomUI_setEnabled(ui,28,simStopped,true)

        simExtCustomUI_setEnabled(ui,1000,bwUtils.getReferencedObjectHandle(model,REF_MASTER_CONV)==-1,true) -- enable
        simExtCustomUI_setEnabled(ui,10,bwUtils.getReferencedObjectHandle(model,REF_MASTER_CONV)==-1,true) -- vel
        simExtCustomUI_setEnabled(ui,12,bwUtils.getReferencedObjectHandle(model,REF_MASTER_CONV)==-1,true) -- accel
        
        simExtCustomUI_setEnabled(ui,100,simStopped,true) -- stop trigger
        simExtCustomUI_setEnabled(ui,101,simStopped,true) -- restart trigger
        simExtCustomUI_setEnabled(ui,102,simStopped,true) -- master
    end
end

function roundedEndsClicked(ui,id,newVal)
    local conf=readInfo()
    conf['bitCoded']=simBoolOr32(conf['bitCoded'],16)
    if newVal~=0 then
        conf['bitCoded']=conf['bitCoded']-16
    end
    writeInfo(conf)
    modified=true
    updateConveyor()
    updateEnabledDisabledItems()
end

function texturedClicked(ui,id,newVal)
    local conf=readInfo()
    conf['bitCoded']=simBoolOr32(conf['bitCoded'],32)
    if newVal~=0 then
        conf['bitCoded']=conf['bitCoded']-32
    end
    writeInfo(conf)
    modified=true
    updateConveyor()
end

function enabledClicked(ui,id,newVal)
    local conf=readInfo()
    conf['bitCoded']=simBoolOr32(conf['bitCoded'],64)
    if newVal==0 then
        conf['bitCoded']=conf['bitCoded']-64
    end
    writeInfo(conf)
    modified=true
end

function redChange(ui,id,newVal)
    modified=true
    local r,g,b,s=getColor()
    setColor(newVal/100,g,b,s)
end

function greenChange(ui,id,newVal)
    modified=true
    local r,g,b,s=getColor()
    setColor(r,newVal/100,b,s)
end

function blueChange(ui,id,newVal)
    modified=true
    local r,g,b,s=getColor()
    setColor(r,g,newVal/100,s)
end

function specularChange(ui,id,newVal)
    modified=true
    local r,g,b,s=getColor()
    setColor(r,g,b,newVal/100)
end

function speedChange(ui,id,newVal)
    local c=readInfo()
    local v=tonumber(newVal)
    if v then
        v=v*0.001
        if v<-0.5 then v=-0.5 end
        if v>0.5 then v=0.5 end
        if v~=c['velocity'] then
            modified=true
            c['velocity']=v
            writeInfo(c)
        end
    end
    simExtCustomUI_setEditValue(ui,10,string.format("%.0f",c['velocity']/0.001),true)
end

function accelerationChange(ui,id,newVal)
    local c=readInfo()
    local v=tonumber(newVal)
    if v then
        v=v*0.001
        if v<0.001 then v=0.001 end
        if v>1 then v=1 end
        if v~=c['acceleration'] then
            modified=true
            c['acceleration']=v
            writeInfo(c)
        end
    end
    simExtCustomUI_setEditValue(ui,12,string.format("%.0f",c['acceleration']/0.001),true)
end

function createDlg()
    if (not ui) and bwUtils.canOpenPropertyDialog() then
        local xml = [[
    <tabs id="77">
    <tab title="General" layout="form">
                <label text="Enabled"/>
                <checkbox text="" onchange="enabledClicked" id="1000"/>

                <label text="Speed (mm/s)"/>
                <edit oneditingfinished="speedChange" id="10"/>

                <label text="Acceleration (mm/s^2)"/>
                <edit oneditingfinished="accelerationChange" id="12"/>

                <label text="Master conveyor"/>
                <combobox id="102" onchange="masterChange_callback">
                </combobox>

                <label text="Stop on trigger"/>
                <combobox id="100" onchange="triggerStopChange_callback">
                </combobox>

                <label text="Restart on trigger"/>
                <combobox id="101" onchange="triggerStartChange_callback">
                </combobox>

            <label text="" style="* {margin-left: 150px;}"/>
            <label text="" style="* {margin-left: 150px;}"/>

    </tab>
    <tab title="Dimensions" layout="form">
                <label text="Length (mm)"/>
                <edit oneditingfinished="lengthChange" id="2"/>

                <label text="Width (mm)"/>
                <edit oneditingfinished="widthChange" id="4"/>

                <label text="Height (mm)"/>
                <edit oneditingfinished="heightChange" id="20"/>

                <label text="Border height (mm)"/>
                <edit oneditingfinished="borderHeightChange" id="21"/>

                <label text="Wall thickness (mm)"/>
                <edit oneditingfinished="wallThicknessChange" id="26"/>
    </tab>
    <tab title="Color" layout="form">
                <label text="Red"/>
                <hslider minimum="0" maximum="100" onchange="redChange" id="5"/>
                <label text="Green"/>
                <hslider minimum="0" maximum="100" onchange="greenChange" id="6"/>
                <label text="Blue"/>
                <hslider minimum="0" maximum="100" onchange="blueChange" id="7"/>
                <label text="Specular"/>
                <hslider minimum="0" maximum="100" onchange="specularChange" id="8"/>
    </tab>
    <tab title="More" layout="form">
                <label text="Textured"/>
                <checkbox text="" onchange="texturedClicked" id="28"/>

                <label text="Left side open"/>
                <checkbox text="" onchange="leftSideOpenClicked" id="22"/>

                <label text="Right side open"/>
                <checkbox text="" onchange="rightSideOpenClicked" id="23"/>

                <label text="Rounded ends"/>
                <checkbox text="" onchange="roundedEndsClicked" id="27"/>

                <label text="Front side open"/>
                <checkbox text="" onchange="frontSideOpenClicked" id="24"/>

                <label text="Back side open"/>
                <checkbox text="" onchange="backSideOpenClicked" id="25"/>
    </tab>
    </tabs>
        ]]
        ui=bwUtils.createCustomUi(xml,simGetObjectName(model),previousDlgPos--[[,closeable,onCloseFunction,modal,resizable,activate,additionalUiAttribute--]])

        local red,green,blue,spec=getColor()
        local config=readInfo()

        simExtCustomUI_setEditValue(ui,2,string.format("%.0f",config['length']/0.001),true)
        simExtCustomUI_setEditValue(ui,4,string.format("%.0f",config['width']/0.001),true)
        simExtCustomUI_setEditValue(ui,20,string.format("%.0f",config['height']/0.001),true)
        simExtCustomUI_setEditValue(ui,21,string.format("%.0f",config['borderHeight']/0.001),true)
        simExtCustomUI_setEditValue(ui,26,string.format("%.0f",config['wallThickness']/0.001),true)

        simExtCustomUI_setCheckboxValue(ui,22,(simBoolAnd32(config['bitCoded'],1)~=0) and 2 or 0,true)
        simExtCustomUI_setCheckboxValue(ui,23,(simBoolAnd32(config['bitCoded'],2)~=0) and 2 or 0,true)
        simExtCustomUI_setCheckboxValue(ui,24,(simBoolAnd32(config['bitCoded'],4)~=0) and 2 or 0,true)
        simExtCustomUI_setCheckboxValue(ui,25,(simBoolAnd32(config['bitCoded'],8)~=0) and 2 or 0,true)
        simExtCustomUI_setCheckboxValue(ui,27,(simBoolAnd32(config['bitCoded'],16)==0) and 2 or 0,true)
        simExtCustomUI_setCheckboxValue(ui,28,(simBoolAnd32(config['bitCoded'],32)==0) and 2 or 0,true)

        simExtCustomUI_setCheckboxValue(ui,1000,(simBoolAnd32(config['bitCoded'],64)~=0) and 2 or 0,true)
        simExtCustomUI_setEditValue(ui,10,string.format("%.0f",config['velocity']/0.001),true)
        simExtCustomUI_setEditValue(ui,12,string.format("%.0f",config['acceleration']/0.001),true)
--        simExtCustomUI_setEditValue(ui,14,string.format("%.0f",config[3]/0.001),true)

        simExtCustomUI_setSliderValue(ui,5,red*100,true)
        simExtCustomUI_setSliderValue(ui,6,green*100,true)
        simExtCustomUI_setSliderValue(ui,7,blue*100,true)
        simExtCustomUI_setSliderValue(ui,8,spec*100,true)

        updateStartStopTriggerComboboxes()
        updateMasterCombobox()

        updateEnabledDisabledItems()
        simExtCustomUI_setCurrentTab(ui,77,dlgMainTabIndex,true)

--]]       
    end
end

function showDlg()
    if not ui then
        createDlg()
    end
end

function removeDlg()
    if ui then
        local x,y=simExtCustomUI_getPosition(ui)
        previousDlgPos={x,y}
        dlgMainTabIndex=simExtCustomUI_getCurrentTab(ui,77)
        simExtCustomUI_destroy(ui)
        ui=nil
    end
end

if (sim_call_type==sim_customizationscriptcall_initialization) then
    REF_STOP_SIG=1 -- index of referenced stop trigger object handle
    REF_START_SIG=2 -- index of referenced start trigger object handle
    REF_MASTER_CONV=3 -- index of referenced master conveyor object handle
    modified=false
    dlgMainTabIndex=0
    lastT=simGetSystemTimeInMs(-1)
    version=simGetInt32Parameter(sim_intparam_program_version)
    revision=simGetInt32Parameter(sim_intparam_program_revision)
    model=simGetObjectAssociatedWithScript(sim_handle_self)
    _MODELVERSION_=0
    _CODEVERSION_=0
    local _info=readInfo()
    bwUtils.checkIfCodeAndModelMatch(model,_CODEVERSION_,_info['version'])
    -- Following for backward compatibility:
    if _info['stopTrigger'] then
        bwUtils.setReferencedObjectHandle(model,REF_STOP_SIG,getObjectHandle_noErrorNoSuffixAdjustment(_info['stopTrigger']))
        _info['stopTrigger']=nil
    end
    if _info['startTrigger'] then
        bwUtils.setReferencedObjectHandle(model,REF_START_SIG,getObjectHandle_noErrorNoSuffixAdjustment(_info['startTrigger']))
        _info['startTrigger']=nil
    end
    if _info['masterConveyor'] then
        bwUtils.setReferencedObjectHandle(model,REF_MASTER_CONV,getObjectHandle_noErrorNoSuffixAdjustment(_info['masterConveyor']))
        _info['masterConveyor']=nil
    end
    ----------------------------------------
    writeInfo(_info)
    
    rotJoints={}
    rotJoints[1]=simGetObjectHandle('genericConveyorTypeA_jointB')
    rotJoints[2]=simGetObjectHandle('genericConveyorTypeA_jointC')

    middleParts={}
    middleParts[1]=simGetObjectHandle('genericConveyorTypeA_sides')
    middleParts[2]=simGetObjectHandle('genericConveyorTypeA_textureA')
    middleParts[3]=simGetObjectHandle('genericConveyorTypeA_forwarderA')
    
    endParts={}
    endParts[1]=simGetObjectHandle('genericConveyorTypeA_textureB')
    endParts[2]=simGetObjectHandle('genericConveyorTypeA_textureC')
    endParts[3]=simGetObjectHandle('genericConveyorTypeA_B')
    endParts[4]=simGetObjectHandle('genericConveyorTypeA_C')
    endParts[5]=simGetObjectHandle('genericConveyorTypeA_forwarderB')
    endParts[6]=simGetObjectHandle('genericConveyorTypeA_forwarderC')

    sides={}
    sides[1]=simGetObjectHandle('genericConveyorTypeA_leftSide')
    sides[2]=simGetObjectHandle('genericConveyorTypeA_rightSide')
    sides[3]=simGetObjectHandle('genericConveyorTypeA_frontSide')
    sides[4]=simGetObjectHandle('genericConveyorTypeA_backSide')

    textureHolder=simGetObjectHandle('genericConveyorTypeA_textureHolder')

	simSetScriptAttribute(sim_handle_self,sim_customizationscriptattribute_activeduringsimulation,true)
    updatePluginRepresentation()
    previousDlgPos,algoDlgSize,algoDlgPos,distributionDlgSize,distributionDlgPos,previousDlg1Pos=bwUtils.readSessionPersistentObjectData(model,"dlgPosAndSize")
end

showOrHideUiIfNeeded=function()
    local s=simGetObjectSelection()
    if s and #s>=1 and s[#s]==model then
        showDlg()
    else
        removeDlg()
    end
end

if (sim_call_type==sim_customizationscriptcall_nonsimulation) then
    showOrHideUiIfNeeded()
    if simGetSystemTimeInMs(lastT)>3000 then
        lastT=simGetSystemTimeInMs(-1)
        if modified then
            simAnnounceSceneContentChange() -- to have an undo point
            modified=false
        end
    end
end

if (sim_call_type==sim_customizationscriptcall_simulationsensing) then
    if simJustStarted then
        updateEnabledDisabledItems()
    end
    simJustStarted=nil
    showOrHideUiIfNeeded()
end

if (sim_call_type==sim_customizationscriptcall_simulationpause) then
    showOrHideUiIfNeeded()
end

if (sim_call_type==sim_customizationscriptcall_firstaftersimulation) then
    updateEnabledDisabledItems()
    local conf=readInfo()
    conf['encoderDistance']=0
    conf['stopRequests']={}
    writeInfo(conf)
end

if (sim_call_type==sim_customizationscriptcall_lastbeforesimulation) then
    simJustStarted=true
end

if (sim_call_type==sim_customizationscriptcall_lastbeforeinstanceswitch) then
    removeDlg()
    removeFromPluginRepresentation()
end

if (sim_call_type==sim_customizationscriptcall_firstafterinstanceswitch) then
    updatePluginRepresentation()
end

if (sim_call_type==sim_customizationscriptcall_cleanup) then
    removeDlg()
    removeFromPluginRepresentation()
    bwUtils.writeSessionPersistentObjectData(model,"dlgPosAndSize",previousDlgPos,algoDlgSize,algoDlgPos,distributionDlgSize,distributionDlgPos,previousDlg1Pos)
end